Friend Class MachineDefinition
    Inherits MachineDefinitionBase
    Implements IMachineDefinition_2_1
	
	''' <summary>
	''' The version of this machine definition as shown by choosing the Version command from the Help menu of the Configure Machine Definition dialog
	''' </summary>
	Private version As System.Version =  New System.Version("1.2.0")
	
    ''' <summary>
    ''' Local variable to store the format for the number of decimal places after the point.
    ''' </summary>
    Private format As String
	
	''' <summary>
	''' Local variable that is used to store the configuration.
	''' </summary>
	''' <remarks>Any changes made by the user from the 'Configure Machine Definition' command are assigned to this variable.</remarks>
	Private _configuration As Configuration
	
#Region "Events"

	''' <summary>
	''' This event handler is invoked when this Machine Definition has finished compiling.
	''' </summary>
	''' <param name="e">Provides data for the compiled event.</param>
    ''' <remarks>Here is where you should perform any one time initialization.</remarks>
	Private Sub Complied(ByVal e As CompiledEventArgs) Implements IMachineDefinition_2_1.Compiled

		UserConfiguredInputDialog.DeleteAllInputBoxes()
        UserConfiguredInputDialog.AddInputBox("Program Number", 50, "1")
        UserConfiguredInputDialog.AddInputBox("'E' Fixture Offset", 50, "1")
		
	End Sub

    ''' <summary>
    ''' This event handler is invoked first when the Nc code is generated and is where you
	''' configure the general settings.
    ''' </summary>
    ''' <param name="e">Provides data for the general settings event.</param>
    Private Sub GeneralSettings(ByVal e As GeneralSettingsEventArgs) Implements IMachineDefinition_2_1.GeneralSettings

        e.OutputLineNumbers = True
        e.StartLineNumber = _configuration.LineNumberIncrement
        e.LineNumberIncrement = _configuration.LineNumberIncrement
        e.ArcBreak = ArcBreak.DoNotBreak
		e.ChipToChipAtcTime = _configuration.ChipToChipAtcTime
		e.RapidPositionAccelerationDeceleration = _configuration.RapidPositionAccelerationDeceleration
		e.SubRoutinesAtStart = True	
		e.SubRoutinesStartNumber = 1
		e.SubRoutinesNumberIncrement = 1
		e.LineNumbering = LineNumbering.Sequentially

        'Metric parts need 3 decimal places after the point, imperial 4
        If e.Metric Then
            format = "0.0##"
            e.MinimumArcLength = 0.005
            e.MinimumArcRadius = 0.005
            e.MaximumArcRadius = 10000
        Else
            format = "0.0###"
            e.MinimumArcLength = 0.0002
            e.MinimumArcRadius = 0.0002
            e.MaximumArcRadius = 400
     		e.RapidTraverseRate = _configuration.RapidTraverseRate / 25.4
		End If

        programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
        fixtureOffset = UserConfiguredInputDialog.GetInputBoxValue("'E' Fixture Offset")

	End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the begining of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileStart(ByVal e As FileStartEventArgs) Implements IMachineDefinition_2_1.FileStart
        
		NcCode.NewString("%")
        NcCode.NewLine()

        'Output the program number and the name of the part
        NcCode.LineNumber("N")
        NcCode.NewString("O" & programNumber & " (" & e.SharpCamFilename.ToUpper() & ")")
        NcCode.NewLine()
		
    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the program can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create any preparatory code.</remarks>
    Private Sub ProgramStart(ByVal e As ProgramStartEventArgs) Implements IMachineDefinition_2_1.ProgramStart


		If _configuration.OutputCycleTime Then
			NcCode.LineNumber("N")
			NcCode.NewString("(CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES)")
			NcCode.NewLine()
		End If

        'List all tools used in this program
        If e.ToolNames.Count > 0 Then
            For Each name As String In e.ToolNames
                NcCode.LineNumber("N")
                NcCode.NewString("(" & name.ToUpper() & ")")
                NcCode.NewLine()
            Next
        End If

        NcCode.LineNumber("N")
        NcCode.NewString("G40 G90 G17 G80")
        NcCode.NewLine()
		If _configuration.OutputG28BeforeATC
        	NcCode.LineNumber("N")
 	    	NcCode.NewString("G91 G28 Z0")			
			NcCode.NewLine()    
		End If
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the tool change event.</param>
    Private Sub ToolChange(ByVal e As ToolChangeEventArgs) Implements IMachineDefinition_2_1.ToolChange

       	NcCode.LineNumber("N")
        NcCode.NewString("T" & e.ToolNumber & " M6 (" & e.ToolName.ToUpper() & ")")
        NcCode.NewLine()
		If _configuration.OutputM01ForATC = M01Type.OutputAfterATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
	        NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked before every tool change, unless it is the first, where it is not called.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel tool change event.</param>
    Private Sub CancelToolChange(ByVal e As CancelToolChangeEventArgs) Implements IMachineDefinition_2_1.CancelToolChange

		If _configuration.OutputG28BeforeATC
        	NcCode.LineNumber("N")
 	   		NcCode.NewString("G91 G28 Z0 M9")			
        	NcCode.NewLine()
		Else
        	NcCode.LineNumber("N")
        	NcCode.NewString("M09")
			NcCode.NewLine()			
		End If
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked for the first position after a tool change for Profiling and Pocketing operations.
    ''' </summary>
    ''' <param name="e">Provides data for the first position after tool change event.</param>
    Private Sub FirstPositionAfterToolChange(ByVal e As FirstPositionAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.FirstPositionAfterToolChange

        NcCode.LineNumber("N")
        NcCode.NewWord("G0", Modal.Off)
        NcCode.NewString("G90 E" & fixtureOffset & " ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewString(e.ToolDirection & " S" & e.SpindleSpeed)
        NcCode.NewLine()
        NcCode.LineNumber("N")
        NcCode.NewString("G43 ")
        NcCode.NewDimensionedWord("Z", e.PositionZ.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewString("H" & e.ToolNumber & " " & e.Coolant)
        NcCode.NewLine()


    End Sub

    ''' <summary>
    ''' This event handler is invoked for all Profiling and Pocketing rapid positions, other than
    ''' the <see cref="FirstPositionAfterToolChange"/>.
    ''' </summary>
    ''' <param name="e">Provides data for the rapid position event.</param>
    Private Sub RapidPositiong(ByVal e As RapidPositionEventArgs) Implements IMachineDefinition_2_1.RapidPositioning

        NcCode.LineNumber("N")
        NcCode.NewWord("G0", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every linear interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the linear interpolation event.</param>
    Private Sub LinearInterpolation(ByVal e As LinearInterpEventArgs) Implements IMachineDefinition_2_1.LinearInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the left.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC left event.</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves.</remarks>
    Private Sub ApplyCRCLeft(ByVal e As ApplyCRCLeftEventArgs) Implements IMachineDefinition_2_1.ApplyCRCLeft

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G41 ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewString("D" & e.ToolOffsetNumber & " ")
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the right.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC right event</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves</remarks>
    Private Sub ApplyCRCRight(ByVal e As ApplyCRCRightEventArgs) Implements IMachineDefinition_2_1.ApplyCRCRight

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G42 ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewString("D" & e.ToolOffsetNumber & " ")
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is canceled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel CRC right event.</param>
    ''' <remarks>This is a linear move. CRC is not cancelled on circular interpolation moves.</remarks>
    Private Sub CancelCRC(ByVal e As CancelCRCEventArgs) Implements IMachineDefinition_2_1.CancelCRC

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G40 ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every counter clockwise circular interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the clockwise circular interpolation event</param>
    Private Sub CWInterpolation(ByVal e As CWInterpolationEventArgs) Implements IMachineDefinition_2_1.CWInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G2", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every clockwise circular interpolation move
    ''' </summary>
    ''' <param name="e">Provides data for the circular interpolation event.</param>
    Private Sub CCWInterpolation(ByVal e As CCWInterpolationEventArgs) Implements IMachineDefinition_2_1.CCWInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G3", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole after a tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole after toolchange event</param>
    Private Sub DrillFirstHoleAfterToolChange(ByVal e As DrillFirstHoleAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleAfterToolChange

        NcCode.LineNumber("N")
        NcCode.NewWord("G0", Modal.Off)
        NcCode.NewString("G90 E" & fixtureOffset & " ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)

        'If we are NOT rigid tapping then start the spindle
        If e.DrillingType = DrillingType.RigidTapping Then
            NcCode.NewString("M5")			
		Else
            NcCode.NewString(e.ToolDirection)			
        	NcCode.NewString(" S" & e.SpindleSpeed)
		End If
        NcCode.NewLine()

        NcCode.LineNumber("N")
        NcCode.NewString("G43 ")
        NcCode.NewDimensionedWord("Z", e.IntialRapid.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewString("H" & e.ToolNumber & " " & e.Coolant)
        NcCode.NewLine()

        NcCode.LineNumber("N")
        Dim feedrate As Double
        If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
            NcCode.NewString("G81 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
            NcCode.NewString("G82 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.PeckDrill Then
            NcCode.NewString("G83 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("G73 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Tapping Then
            NcCode.NewString("G84 ")
            feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
        ElseIf e.DrillingType = DrillingType.RigidTapping Then
            NcCode.NewWord("G84.1", Modal.Off)
            feedrate = e.Pitch * e.SpindleSpeed
        ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
            NcCode.NewString("G85 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
			'Add code for additional drilling cycle
		End If
        Dim returnType As String
        If e.RetractToIntialRapid Then
            returnType = "G98"
        Else
            returnType = "G99"
        End If
        NcCode.NewString(returnType & " R" & e.FeedFrom.ToString(format, CultureInfo) & " ")
        NcCode.NewDimensionedWord("Z", e.FinishDepth.ToString(format, CultureInfo), Modal.Off)
        If (e.DrillingType = DrillingType.Drilling) And e.Dwell > 0 Then
            NcCode.NewString("P" & e.Dwell * 1000 & " ")
        ElseIf e.DrillingType = DrillingType.PeckDrill Or e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("P" & (_configuration.PWordForPeck * e.FeedRate/e.SpindleSpeed).ToString(format, CultureInfo) & " ")
            NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
        ElseIf e.DrillingType = DrillingType.RigidTapping Then
			NcCode.NewString("S" & e.SpindleSpeed & " ")
		End If
        NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()
		
		If e.RetractToIntialRapid Then
        	NcCode.UpdateDimensionedWord("Z", e.IntialRapid)
		Else
        	NcCode.UpdateDimensionedWord("Z", e.FeedFrom)
        End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole in an operation that is NOT after a Tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole in operation event.</param>
    Private Sub DrillFirstHoleInOperation(ByVal e As DrillFirstHoleInOperationEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleInOperation

        NcCode.LineNumber("N")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.IntialRapid.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()

        NcCode.LineNumber("N")
		Dim feedrate As Double
        If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
            NcCode.NewString("G81 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
            NcCode.NewString("G82 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.PeckDrill Then
            NcCode.NewString("G83 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("G73 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Tapping Then
            NcCode.NewString("G84 ")
            feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
        ElseIf e.DrillingType = DrillingType.RigidTapping Then
            NcCode.NewWord("G84.1", Modal.Off)
            feedrate = e.Pitch * e.SpindleSpeed
        ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
            NcCode.NewString("G85 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
			'Add code for additional drilling cycle
		End If
        Dim returnType As String
        If e.RetractToIntialRapid Then
            returnType = "G98"
        Else
            returnType = "G99"
        End If
        NcCode.NewString(returnType & " R" & e.FeedFrom.ToString(format, CultureInfo) & " ")
        NcCode.NewDimensionedWord("Z", e.FinishDepth.ToString(format, CultureInfo), Modal.Off)
        If (e.DrillingType = DrillingType.Drilling) And e.Dwell > 0 Then
            NcCode.NewString("P" & e.Dwell * 1000 & " ")
        ElseIf e.DrillingType = DrillingType.PeckDrill Or e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("P" & (_configuration.PWordForPeck * e.FeedRate/e.SpindleSpeed).ToString(format, CultureInfo) & " ")
            NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
        End If
        NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()
	
		If e.RetractToIntialRapid Then
        	NcCode.UpdateDimensionedWord("Z", e.IntialRapid)
		Else
        	NcCode.UpdateDimensionedWord("Z", e.FeedFrom)
        End If
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill a hole that is NOT after a Tool change or the first in the operation.
    ''' </summary>
    ''' <param name="e">Provides data for the drill hole event.</param>
    Private Sub DrillHole(ByVal e As DrillHoleEventArgs) Implements IMachineDefinition_2_1.DrillHole

        NcCode.LineNumber("N")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when a drilling cycle needs to be canelled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel drilling cycle event.</param>
    Private Sub CancelDrillingCycles(ByVal e As CancelDrillingCyclesEventArgs) Implements IMachineDefinition_2_1.CancelDrillingCycles

		NcCode.LineNumber("N")
        NcCode.NewString("G80")
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked last.
    ''' </summary>
    Private Sub ProgramEnd(ByVal e As ProgramEndEventArgs) Implements IMachineDefinition_2_1.ProgramEnd

        NcCode.LineNumber("N")
        NcCode.NewString("G91 G28 Z0")
        NcCode.NewLine()
        NcCode.LineNumber("N")
        NcCode.NewString("G90")
        NcCode.NewLine()
        NcCode.LineNumber("N")
		If e.Metric
        	NcCode.NewString("G53 Y" & (_configuration.YAxisStroke / 2 * 25.4).ToString(format, CultureInfo))
		Else
        	NcCode.NewString("G53 Y" & (_configuration.YAxisStroke / 2).ToString(format, CultureInfo))
		End If
        NcCode.NewLine()
        NcCode.LineNumber("N")
        NcCode.NewString("M30")
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the end of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileEnd(ByVal e As FileEndEventArgs) Implements IMachineDefinition_2_1.FileEnd
        
		NcCode.NewString("%")
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineAllStart(ByVal e As SubRoutineAllStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllStart

	End Sub

    ''' <summary>
    ''' This event handler is invoked when a Sub Routine is called.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineCall(ByVal e As SubRoutineCallEventArgs) Implements IMachineDefinition_2_1.SubRoutineCall

		NcCode.LineNumber("N")
        NcCode.NewString("L" & e.SubRoutineNumber * 100 + 1)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine start event.</param>
    Private Sub SubRoutineStart(ByVal e As SubRoutineStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineStart
        
		NcCode.LineNumber("N")
		NcCode.NewString("L" & e.SubRoutineNumber * 100)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine end event.</param>
    Private Sub SubRoutineEnd(ByVal e As SubRoutineEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineEnd

		NcCode.LineNumber("N")
		NcCode.NewString("M17")
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine end event.</param>
    Private Sub SubRoutineAllEnd(ByVal e As SubRoutineAllEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllEnd

		NcCode.LineNumber("N")
		NcCode.NewString("M30")
        NcCode.NewLine()

	End Sub

	''' <summary>
	''' <summary>
	''' This event handler is invoked so a program stop/comment can be created.
	''' </summary>
	''' <param name="e">Provides data for the program stop event.</param>
	''' <remarks>Here is where you should create all the logic for a program stop, typlically M00/M01.</remarks>
	Private Sub ProgramStop(ByVal e As ProgramStopEventArgs) Implements IMachineDefinition_2_1.ProgramStop

		If e.ProgramStopType = ProgramStopType.Stop Then
			NcCode.LineNumber("N")
			NcCode.NewString("M0")
			NcCode.NewLine()			
		Else If e.ProgramStopType = ProgramStopType.Optional Then
			NcCode.LineNumber("N")
			NcCode.NewString("M1")
			NcCode.NewLine()			
		End If
		For Each commentLine As String In  e.Comment
                NcCode.LineNumber("N")
				If commentLine.Length > 0 Then
                	NcCode.NewString("(" & commentLine.ToUpper() & ")")				
				End If
                NcCode.NewLine()
        Next
	
	End Sub

	''' <summary>
	''' This event handler is invoked at the beginning of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the start interpolation event.</param>
	''' <remarks>Used for turning on the torch when using Jet cutting.</remarks>
	Private Sub StartInterpolation(ByVal e As StartInterpolationEventArgs) Implements IMachineDefinition_2_1.StartInterpolation

	End Sub

	''' <summary>
	''' This event handler is invoked at the end of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the end interpolation event.</param>
	''' <remarks>Used for turning off the torch when using Jet cutting.</remarks>
	Private Sub EndInterpolation(ByVal e As EndInterpolationEventArgs) Implements IMachineDefinition_2_1.EndInterpolation

	End Sub

#End Region

#Region "User Congigured Input"

    ''' <summary>
    ''' A String varable to store the program number, this is available through out the
    ''' Machine Definition
    ''' </summary>
    Private programNumber As String

    ''' <summary>
    ''' A String varable to store the fixture offset, this is available through out the
    ''' Machine Definition
    ''' </summary>
    Private fixtureOffset As String

    ''' <summary>
    ''' When the user clicks the 'User Configured Input' button on the NC Code tab in the Part Manager,
    ''' this sub procedure is called. Here we display the dialog box where the user can input information
	''' that can be used in the Machine Definition.
    ''' </summary>
    Private Sub UserConfiguredInput(ByVal e As UserConfiguredInputEventArgs) Implements IMachineDefinition_2_1.UserConfiguredInput

        UserConfiguredInputDialog.Display()
        programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
        fixtureOffset = UserConfiguredInputDialog.GetInputBoxValue("Fixture Offset")
        NcCode.GenerateCode()

    End Sub

#End Region

#Region "Properties"

    ''' <summary>
    ''' Gets the Nc code to be output for a clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CWSpindleStart() As String Implements IMachineDefinition_2_1.CWSpindleStart
        Get
            Return "M3"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for a counter clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CCWSpindleStart() As String Implements IMachineDefinition_2_1.CCWSpindleStart
        Get
            Return "M4"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for flood coolant.
    ''' </summary>
    Private ReadOnly Property CoolantFlood() As String Implements IMachineDefinition_2_1.CoolantFlood
        Get
			If _configuration.CoolantSetup = CoolantSetup.M8FloodM7ThroughSpindle
  	          Return "M8"
			Else
  	          Return "M7"				
			End If
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through spindle.
    ''' </summary>
    Private ReadOnly Property CoolantThroughSpindle() As String Implements IMachineDefinition_2_1.CoolantThroughSpindle
        Get
			If _configuration.CoolantSetup = CoolantSetup.M8FloodM7ThroughSpindle
  	          Return "M7"				
			Else
  	          Return "M8"
			End If
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through the tool.
    ''' </summary>
    Private ReadOnly Property CoolantThroughTool() As String Implements IMachineDefinition_2_1.CoolantThroughTool
        Get
            Return "M52"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for mist coolant.
    ''' </summary>
    Private ReadOnly Property CoolantMist() As String Implements IMachineDefinition_2_1.CoolantMist
        Get
            Return "M7"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for an air blast.
    ''' </summary>
    Private ReadOnly Property CoolantAirBlast() As String Implements IMachineDefinition_2_1.CoolantAirBlast
        Get
            Return "M53"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant off.
    ''' </summary>
    Private ReadOnly Property CoolantNone() As String Implements IMachineDefinition_2_1.CoolantNone
        Get
            Return "M9"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for custom coolant.
    ''' </summary>
    Private ReadOnly Property CoolantCustom() As String Implements IMachineDefinition_2_1.CoolantCustom
        Get
            Return "M9"
        End Get
    End Property

#End Region

#Region "Modals"

    ''' <summary>
    ''' This event handler is invoked to sets up modal words
    ''' </summary>
    ''' <param name="e">Provides data for the modals event</param>
    Private Sub Modals(ByVal e As ModalEventArgs) Implements IMachineDefinition_2_1.Modals

        'Firstly create a new 'ModalWordGroup'.
        Dim wordGroup1 As New ModalWordGroup()
        'Add Words to 'wordGroup1'.
        wordGroup1.Words.Add("G0")
        wordGroup1.Words.Add("G1")
        wordGroup1.Words.Add("G2")
        wordGroup1.Words.Add("G3")

        'Finally add the groups to 'WordGroups'.
        e.WordGroups.Add(wordGroup1)

        'Add Absolute Modal Dimentioned Words to 'AbsoluteDimensionedWords'.	
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("X"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Y"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Z"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("F"))

        'Add Incremental Modal Dimentioned Words to 'IncrementalDimensionedWords'.		
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("I"))
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("J"))

    End Sub

#End Region

End Class

#Region "Configuration"

''' <summary>
''' Class to manage the configuration of this Machine Definition.
''' </summary>
Public Class Configuration
	
	Private _toolChangerType As ToolChangerType
	Private _yAxisStroke As Double
	Private _lineNumberIncrement As Double
	Private _coolantSetup As CoolantSetup
	Private _outputG28BeforeATC As Boolean
	Private _outputM01ForATC As M01Type
	Private _pWordForPeck As Double
	Private _outputCycleTime As Boolean
	Private _chipToChipAtcTime As Double
	Private _rapidTraverseRate As Double
	Private _rapidPositionAccelerationDeceleration As Double
	''' <summary>
	''' Gets or sets the type of tool changer.
	''' </summary>
	<System.ComponentModel.Description("Choose 'Umbrella' if your machine does not pre select tools. Choose 'DualArm' if your machine pre selects tools."), _
	System.ComponentModel.DisplayName("Type of Tool changer"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property ToolChangerType() As ToolChangerType
        Get
            Return _toolChangerType
        End Get
        Set
            _toolChangerType = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the position from the refernce point that the Y axis is positioned at the end of the program.
	''' </summary>
	<System.ComponentModel.Description("The stroke of the Y axis in inches. Typically the model indicated the Y axis stroke. eg a 4020 has a 20 inch stroke. This value is used to position the Y axis for unloading."), _
	System.ComponentModel.DisplayName("Y axis stroke")> _
	Public Property YAxisStroke() As Double
        Get
            Return _yAxisStroke
        End Get
        Set
            _yAxisStroke = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the line number increment.
	''' </summary>
	<System.ComponentModel.Description("The increment value for the next line number."), _
	System.ComponentModel.DisplayName("Line Number Increment")> _
	Public Property LineNumberIncrement() As Double
        Get
            Return _lineNumberIncrement
        End Get
        Set
            _lineNumberIncrement = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the M8 and M7 coolant setting.
	''' </summary>
	<System.ComponentModel.Description("Controls the coolant settings for M8 and M7. This is configured by used the SETP comamnd on the control"), _
	System.ComponentModel.DisplayName("M8 and M7 coolant setting")> _
	Public Property CoolantSetup() As CoolantSetup
        Get
            Return _coolantSetup
        End Get
        Set
            _coolantSetup = value
        End Set
    End Property	
	
	''' <summary>
	''' Gets or sets a value indicating if the Z axis should be returned to the G28 position prior to a toolchange.
	''' </summary>
	<System.ComponentModel.Description("Should the Z axis be returned to the G28 position prior to a toolchange"), _
	System.ComponentModel.DisplayName("Output G28 prior to ATC"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property OutputG28BeforeATC() As Boolean
        Get
            Return _outputG28BeforeATC
        End Get
        Set
            _outputG28BeforeATC = value
        End Set
    End Property	
	
	''' <summary>
	''' Gets or sets a value indicating if and where an M01 is output for an ATC.
	''' </summary>
	<System.ComponentModel.Description("Specifies if and where an M01 is output for an ATC"), _
	System.ComponentModel.DisplayName("Output M01 for ATC"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property OutputM01ForATC() As M01Type
        Get
            Return _outputM01ForATC
        End Get
        Set
            _outputM01ForATC = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating P word amount when using G83 or G73.
	''' </summary>
	<System.ComponentModel.Description("Distance to rapid to within the last peck. This value is a multiple of the feed per revolution. So if the value is set to 5, then the P word value is 5 * feed per revolution."), _
	System.ComponentModel.DisplayName("P word for G83/G73")> _
	Public Property PWordForPeck() As Double
        Get
            Return _pWordForPeck
        End Get
        Set
            _pWordForPeck = value
        End Set
    End Property
	

	''' <summary>
	''' Gets or sets a value indicating if the cycle time should be output in a comment.
	''' </summary>
	<System.ComponentModel.Description("Should the cycle time be output?"), _
	System.ComponentModel.DisplayName("Output Cycle Time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property OutputCycleTime() As Boolean
        Get
            Return _outputCycleTime
        End Get
        Set
            _outputCycleTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the chip to chip tool change time in seconds.
	''' </summary>
	<System.ComponentModel.Description("The chip to chip tool change time in seconds"), _
	System.ComponentModel.DisplayName("Chip to chip tool change time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property ChipToChipAtcTime() As Double
        Get
            Return _chipToChipAtcTime
        End Get
        Set
            _chipToChipAtcTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the rapid traverse rate in millimetres/minute.
	''' </summary>
	<System.ComponentModel.Description("The rapid traverse rate in millimetres/minute. This value is converted to imperial for Parts with Inch units."), _
	System.ComponentModel.DisplayName("Rapid traverse rate"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidTraverseRate() As Double
        Get
            Return _rapidTraverseRate
        End Get
        Set
            _rapidTraverseRate = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration.
	''' </summary>
	<System.ComponentModel.Description("The time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration"), _
	System.ComponentModel.DisplayName("Rapid Acceleration/Deceleration"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidPositionAccelerationDeceleration() As Double
        Get
            Return _rapidPositionAccelerationDeceleration

        End Get
        Set
            _rapidPositionAccelerationDeceleration = value
        End Set
    End Property

End Class

''' <summary>
''' The type of tool changer.
''' </summary>
Public Enum ToolChangerType

	''' <summary>
	''' Umbrella type tool changer fitted.
	''' </summary>
	Umbrella
	
	''' <summary>
	''' Dual Arm type tool changer fitted.
	''' </summary>
	DualArm
	
End Enum

''' <summary>
''' M8 and M7 coolant setting.
''' </summary>
Public Enum CoolantSetup

	''' <summary>
	''' M8 controls the flood coolant and M7 controls the through spindle coolant.
	''' </summary>
	M8FloodM7ThroughSpindle
	
	''' <summary>
	''' M7 controls the flood coolant and M8 controls the through spindle coolant.
	''' </summary>
	M7FloodM8ThroughSpindle
	
End Enum

''' <summary>
''' The position of M01.
''' </summary>
Public Enum M01Type

	''' <summary>
	''' Do not output M01.
	''' </summary>
	DoNotOutput

	''' <summary>
	''' Output M01 before the ATC.
	''' </summary>
	OutputBeforeATC
	
	''' <summary>
	''' Output M01 after the ATC.
	''' </summary>
	OutputAfterATC
	
End Enum

#End Region
